# Overall Notes:

## Filters
I forgot to teach one thing called filters.
- This is how you create declarative looking html without lots of mutation.
- Examine .last-seen in Messages.vue
- Ref: https://vuejs.org/v2/guide/filters.html 

## Layout

Look how the layout is defined locally and recursively.
- In particular, see how App.vue already defines the overall look.

Examine how each of the components define their own localized layout.
- In practice most people will write their own .scss files and reference them here and there.
- However, conceptually, it is nice to have self contained components.
    - In practice, you will still want global variables. to be in their own SCSS files.


## types.ts

I forgot to add in a few other fields in IUser.



# Task 0: Webpack

I reworked webpack to use my own config.

This is probably not important for you, unless you are an intern.

I haven't gotten around to doing asset bundling.
- Leave the static files in public, not src/assets/*

# Task 1: Friendslist

File: components/FriendsBar.vue

Observe the filters for timeago.

Ignore `@click="activateMessages(friend)"` and  `getLastMessage`, Those are for task 3 and task 4 respectively.

Otherwise, its as straight-forward as it gets.


# Task 2: Messages

File: components/Messages.vue

Begin with a straightforward porting of layout.

```
<div class="messages">
    <div class="message" v-for="(message, idx) of currMessages" :key="idx">
    <span class="name"> {{ message.sender | getUserNameById }} </span>
    <p> {{message.msg}} </p>
    </div>
</div>
```

Getting the name is somewhat more challenging:
`getUserNameById` <-- Write a helper function in the corresponding service to do this for you.


# Task 3: Layout integration

## Layout 
First you need to layout everything nicely in App.vue. 
- I'm not going to comment too much on that - that's just porting the css solution over.


## Router
Begin by updating router.ts

```
routes: [
{
    path: '/:id?',
    name: 'home',
    component: Home,
},
],
```

In particular, this the ability to add a id to the home page. See [Dynamic route matching](https://router.vuejs.org/guide/essentials/dynamic-matching.html).

Next you want to trigger route changes when the user clicks on items in the first list.
- It might be tempting to use router-link, but those create `<a>` tags, not very pretty. I tried :3
- Instead we will just add a click handler `@click="activateMessages(friend)"`

```
protected activateMessages(friend: any) {
    // Dynamically trigger routes.
    // https://router.vuejs.org/guide/essentials/navigation.html
    this.$router.push(`/${friend.id}`);
}
```

Finally, we want to detect the route changes, then update the selectedUserId

To do this, we create a bunch of getters:

```
get selectedUserId() {
    if (messages.lastUpdated && this.$route.params.id) {
        return this.$route.params.id;
    }
}

get selectedUser(){
    const id = this.selectedUserId;
    if (id) {
    return _.find(friends.list, (x) => x.id === id);
    }
}

get currMessages() {
    const user = this.selectedUserId;
    if (!user) {
        return [];
    }
    return this.messages.lookup[user];
}
```

# Task 4: Creating a FriendService

Frankly, i just create these things out of habit xD

## lastMessage

This one is quite fun, took me some time. 

I decided to add it create this in Message.ts.

```
export function getLastMessage(user: IUser){
    const id = user.id;

    // NOTE: if you use messages.lookup[id] = [], you WILL NOT get reactive arrays.
    if (!messages.lookup[id]) {
        Vue.set(messages.lookup, id, []);
    }
    const userMessages = messages.lookup[id];
    if (!userMessages.length) {
        return NO_MSGS;
    }
    const lastMessage = userMessages[userMessages.length - 1];
    const prefix = lastMessage.sender === MY_USER_ID ? 'Me: ' : '';
    return `${prefix}${lastMessage.msg}`;
}
```

The key observation is honestly... understanding how reactivity works in Vue.

## seenMessage
Anyway i didn't bother doing the seen messages.

The idea would be to add a timestamp on the message.

Put a userSeen on the FriendService.

Compare lastMessage timestamp to userSeen.

