//Load the .env files -> process.env
const dotenv = require('dotenv');
const env_config = dotenv.config();
const webpack = require('webpack');
const path = require('path');
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');


// TODO: check for need of vue-style-loader?


const STYLE_LOADER = { 
	loader: "style-loader",
	options: {
		sourceMap: true,
	}
};

const CSS_LOADER = { 
	loader: "css-loader",
	options: {
		sourceMap: true,
	}
};

const SASS_LOADER = { 
	loader: "sass-loader",
	options: {
		sourceMap: true,
	}
};

console.log(".env");
console.log(env_config);

// =========== GENERATE ENTRYPOINTS ===========

// Idea
// For each html file, i want to load a JS file with the corresponding name.
// This allows me to create as many html files as there are tasks.

const _ = require('lodash');
const fs = require('fs');
const htmlRegex = /\.html$/;
const filenames = fs.readdirSync(path.join(__dirname, "public")); // Read the current folder
const htmlfiles = filenames
	.filter( x => htmlRegex.exec(x)) // Grab only html files
	.map( x => x.substring(0, x.length - 5)); // strip the .html

const entryPoints = _.zipObject(htmlfiles, 
	htmlfiles.map( x => `./src/${x}.ts`)); 
// Entrypoints are files in the format:
// filename: './src/filename.js'

console.log(entryPoints);


module.exports = {
	entry: entryPoints,
	output: {
		filename: "[name].js",
		path: path.resolve(__dirname, 'dist'),
		publicPath: "/",
	},
	resolve: {
		extensions: [".ts", ".tsx", ".js", ".jsx", ".vue"],
		alias: {
			'@': path.join(__dirname, 'src'),
			vue$: 'vue/dist/vue.runtime.esm.js'
		},
	},
	devServer: {
		historyApiFallback: true
	},
	module: {
		rules: [
			{
				test: /\.vue$/,
				loader: 'vue-loader'
			},
			{
				test: /\.tsx?$/,
				loader: 'ts-loader',
				exclude: /node_modules/,
				// This is required for Vue Projects, and took me 5+ hours to find
				options: { appendTsSuffixTo: [/\.vue$/] } 

			},
			{
			  test: /\.m?js$/,
			  exclude: /(node_modules|bower_components)/,
			  use: {
				loader: 'babel-loader',
				options: {
				  presets: ['@babel/preset-env']
				}
			  }
			},
			{
				test: /\.css$/,
				use: [STYLE_LOADER, CSS_LOADER],
			},
			{
				test: /\.scss$/,
				use: [STYLE_LOADER, CSS_LOADER, SASS_LOADER],
			},
		]
	},
	plugins: [
		new CleanWebpackPlugin(),
		// make sure to include the plugin for the magic
		new VueLoaderPlugin(),
		new HtmlWebpackPlugin({
			template: path.join(__dirname, "public", "index.html"),
			templateParameters: process.env,
		}),
		new CopyPlugin([
			{ from: 'public', to: '.', toType: 'dir' },
		]),
	]
};


